@@ -17,16 +17,16 @@ SET pg_hint_plan.enable_hint_table to on;	-- error
 SET compute_query_id to on;
 SET pg_hint_plan.enable_hint_table to on;
 SET compute_query_id to off;
-SELECT 1;									-- gets warining
+SELECT 1;									-- gets warning
 SELECT 1;									-- not
 SET compute_query_id to on;
 SELECT 1;									-- reactivated
 SET compute_query_id to off;
-SELECT 1;									-- gets warining
+SELECT 1;									-- gets warning
 SET pg_hint_plan.enable_hint_table to off;
 SET compute_query_id to on;
 SET pg_hint_plan.enable_hint_table to on;
-SELECT 1;									-- no warining
+SELECT 1;									-- no warning
 
 RESET compute_query_id;
 RESET pg_hint_plan.enable_hint_table;
@@ -862,11 +862,13 @@ EXPLAIN (COSTS false) SELECT val FROM p1 WHERE val = 1;
 /*+ BitmapScan(p1 p1_.*val2.*)*/
 EXPLAIN (COSTS false) SELECT val FROM p1 WHERE val = 1;
 
--- search from hint table
-INSERT INTO hint_plan.hints (norm_query_string, application_name, hints) VALUES ('EXPLAIN (COSTS false) SELECT * FROM t1 WHERE t1.id = ?;', '', 'SeqScan(t1)');
-INSERT INTO hint_plan.hints (norm_query_string, application_name, hints) VALUES ('EXPLAIN (COSTS false) SELECT id FROM t1 WHERE t1.id = ?;', '', 'IndexScan(t1)');
-INSERT INTO hint_plan.hints (norm_query_string, application_name, hints) VALUES ('EXPLAIN SELECT * FROM t1 WHERE t1.id = ?;', '', 'BitmapScan(t1)');
-SELECT * FROM hint_plan.hints ORDER BY id;
+-- Search from hint table
+SELECT get_query_id('SELECT * FROM t1 WHERE t1.id = 1') AS query_id \gset
+INSERT INTO hint_plan.hints (query_id, application_name, hints)
+  VALUES (:'query_id', '', 'SeqScan(t1)');
+SELECT get_query_id('SELECT id FROM t1 WHERE t1.id = 1') AS query_id \gset
+INSERT INTO hint_plan.hints (query_id, application_name, hints)
+  VALUES (:'query_id', '', 'IndexScan(t1)');
 SET pg_hint_plan.enable_hint_table = on;
 EXPLAIN (COSTS false) SELECT * FROM t1 WHERE t1.id = 1;
 SET pg_hint_plan.enable_hint_table = off;
@@ -1086,79 +1088,65 @@ SELECT val::int FROM p2 WHERE id < 1000;
 /*+ Rows(x) */ SELECT 1;
 
 -- value types
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 #99) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 +99) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 -99) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 *99) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 *0.01) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 #aa) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id); -- ERROR
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 /99) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id); -- ERROR
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
 -- round up to 1
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 -99999) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
 -- complex join tree
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id) JOIN t3 ON (t3.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t2 #22) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id) JOIN t3 ON (t3.id = t2.id);
-\o
-\! sql/maskout.sh results/pg_hint_plan.tmpout
+');
 
-\o results/pg_hint_plan.tmpout
+SELECT explain_filter('
 /*+ Rows(t1 t3 *10) */
 EXPLAIN SELECT * FROM t1 JOIN t2 ON (t1.id = t2.id) JOIN t3 ON (t3.id = t2.id);
-\o
-set max_parallel_workers_per_gather to DEFAULT;
-\! sql/maskout.sh results/pg_hint_plan.tmpout
-\! rm results/pg_hint_plan.tmpout
+');
 
 -- Query with join RTE and outer-join relids
 /*+Leading(ft_1 ft_2 t1)*/
